// avifix, v0.1 -dink 2023
// License: ijwafym-0.c
//
// Compile: gcc avifix.c -o avifix.exe
//
// description:
// Large avi's written by certain Win* emulators using vfw AVIFile API
// will usually have clicks/desyncs at the end, this will fix that problem
// by removing the corrupt \"idx1\" section.
//
// This issue is especially annoying when there are several videos that need to
// be linked together - as there will be a click at each join-point in the
// final video..  After they're linked, ffmpeg will create a fresh new idx1
//
// Note: this really is meant to be used with linkvid.cmd, but, do as you wish


#include <stdio.h>
#include <stdlib.h> // malloc/free()
#include <unistd.h> // ftruncate()

#define MEG (1024*1024)
#define NAME "avifix"

void *memfind(const void *needle, size_t needle_len, const void *haystack, size_t haystack_len) {
	const char *needle_c = (const char *)needle;
	const char *haystack_c = (const char *)haystack;

    for (size_t i = 0; i <= needle_len - haystack_len; i++) {
        size_t j = 0;
        for (j = 0; j < haystack_len; j++) {
            if (needle_c[i + j] != haystack_c[j]) {
                break;
            }
        }

        if (j == haystack_len) {
            return (void *) (needle_c + i);
        }
    }

    return NULL;
}

int err(char *s)
{
	printf("%s", s);

	return 1;
}

int main(int argc, char **argv)
{
	if (argc < 2)
		return err(NAME " v0.1 by dink (2023)\n\nLarge avi's written by certain Win* emulators using vfw AVIFile API\n"
				   "will usually have clicks at the end, this will fix that problem\n"
				   "by removing the corrupt \"idx1\" section.\n\n"
				   "USAGE: " NAME " <filename>\n");

	FILE *fp = fopen(argv[1], "rb+");
	if (!fp) return err(NAME ": can't open file.\n");
	fseek(fp, 0, SEEK_END);
	int len = ftell(fp);

	if (len > MEG * 40) { // avi w/f*'d idx1 is usually > 130megs
		fseek(fp, len - MEG, SEEK_SET);
		char *buf = (char*)malloc(MEG);
		fread(buf, 1, MEG, fp);
		fclose(fp);
		char *pos = (char *)memfind(buf, MEG, "idx1", 4);
		free(buf);

		if (pos == NULL) return err(NAME ": can't find idx1 section, already processed?\n");

		int posi = pos - buf;
		posi = len - (MEG) + posi;
		printf("launching himars strike at idx1, position %x\n", posi);

		fp = fopen(argv[1], "rb+");
		if (!fp) return err("uhoh!\n");

		ftruncate(fileno(fp), posi); // jack had a groove, and it definitely wasn't a broken corrupt idx1
		fclose(fp);
	}
}
